@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
  * gives permission to link the code of this program with
@@ -106,10 +106,15 @@ typedef void             (*EnchantPreConfigureFunc) (EnchantProvider * provider,
 /********************************************************************************/
 /********************************************************************************/
 
+#ifdef _WIN32
+#define path_cmp g_utf8_collate
+#else
+#define path_cmp strcmp
+#endif
 
-static GSList* enchant_slist_prepend_unique_string (GSList *slist, gchar* data)
+static GSList* enchant_slist_prepend_unique_path (GSList *slist, gchar* data)
 {
-	if (NULL == g_slist_find_custom (slist, data, (GCompareFunc)strcmp))
+	if (NULL == g_slist_find_custom (slist, data, (GCompareFunc)path_cmp))
 		{
 			return g_slist_prepend (slist, data);
 		}
@@ -120,9 +125,9 @@ static GSList* enchant_slist_prepend_unique_string (GSList *slist, gchar* data)
 		}
 }
 
-static GSList* enchant_slist_append_unique_string (GSList *slist, gchar* data)
+static GSList* enchant_slist_append_unique_path (GSList *slist, gchar* data)
 {
-	if (NULL == g_slist_find_custom (slist, data, (GCompareFunc)strcmp))
+	if (NULL == g_slist_find_custom (slist, data, (GCompareFunc)path_cmp))
 		{
 			return g_slist_append (slist, data);
 		}
@@ -138,14 +143,15 @@ _enchant_get_user_home_dirs (void)
 {
 	GSList *dirs = NULL;
 	const char* home_dir;
+	char *tmp;
 
-	home_dir = enchant_get_registry_value_ex (1, "Config", "Home_Dir");
-	if (home_dir)
-		dirs = enchant_slist_append_unique_string (dirs, g_strdup (home_dir));
+	tmp = enchant_get_registry_value_ex (1, "Config", "Home_Dir");
+	if (tmp)
+		dirs = enchant_slist_append_unique_path (dirs, tmp);
 
 	home_dir = g_get_home_dir ();
 	if (home_dir)
-		dirs = enchant_slist_append_unique_string (dirs, g_strdup (home_dir));
+		dirs = enchant_slist_append_unique_path (dirs, g_strdup (home_dir));
 
 	return dirs;
 }
@@ -171,7 +177,7 @@ enchant_get_user_dirs (void)
 		user_config_dir = g_get_user_config_dir();
 		
 		if (user_config_dir)
-			user_dirs = enchant_slist_append_unique_string (user_dirs, g_build_filename (user_config_dir,
+			user_dirs = enchant_slist_append_unique_path (user_dirs, g_build_filename (user_config_dir,
 										 "enchant",
 										 NULL));
 	}
@@ -182,7 +188,7 @@ enchant_get_user_dirs (void)
 		
 		for (dir = home_dirs; dir; dir = dir->next)
 			{
-				user_dirs = enchant_slist_append_unique_string (user_dirs,
+				user_dirs = enchant_slist_append_unique_path (user_dirs,
 							    g_build_filename (dir->data,
 									      ENCHANT_USER_PATH_EXTENSION,
 									      NULL));
@@ -209,11 +215,11 @@ enchant_get_module_dirs (void)
 		
 		user_module_dir = enchant_get_registry_value_ex (1, "Config", "Module_Dir");
 		if (user_module_dir)
-			module_dirs = enchant_slist_append_unique_string (module_dirs, user_module_dir);
+			module_dirs = enchant_slist_append_unique_path (module_dirs, user_module_dir);
 	}
 
 #ifdef XP_TARGET_COCOA
-	module_dirs = enchant_slist_append_unique_string (module_dirs, g_strdup ([[EnchantResourceProvider instance] moduleFolder]));
+	module_dirs = enchant_slist_append_unique_path (module_dirs, g_strdup ([[EnchantResourceProvider instance] moduleFolder]));
 #endif
 
 	{
@@ -222,7 +228,7 @@ enchant_get_module_dirs (void)
 		user_dirs = enchant_get_user_dirs();
 
 		for (iter = user_dirs; iter; iter = iter->next)
-			module_dirs = enchant_slist_append_unique_string (module_dirs, iter->data);
+			module_dirs = enchant_slist_append_unique_path (module_dirs, iter->data);
 		
 		g_slist_free (user_dirs);
 	}
@@ -230,19 +236,19 @@ enchant_get_module_dirs (void)
 	/* Look for explicitly set registry values */
 	module_dir = enchant_get_registry_value_ex (0, "Config", "Module_Dir");
 	if (module_dir)
-		module_dirs = enchant_slist_append_unique_string (module_dirs, module_dir);
+		module_dirs = enchant_slist_append_unique_path (module_dirs, module_dir);
 
+#if defined(ENCHANT_GLOBAL_MODULE_DIR)
+	module_dirs = enchant_slist_append_unique_path (module_dirs, g_strdup (ENCHANT_GLOBAL_MODULE_DIR));
+#else
 	/* Dynamically locate library and search for modules relative to it. */
 	prefix = enchant_get_prefix_dir();
 	if(prefix)
 		{
 			module_dir = g_build_filename(prefix,"lib","enchant",NULL);
 			g_free(prefix);
-			module_dirs = enchant_slist_append_unique_string (module_dirs, module_dir);
+			module_dirs = enchant_slist_append_unique_path (module_dirs, module_dir);
 		}
-
-#if defined(ENCHANT_GLOBAL_MODULE_DIR)
-	module_dirs = enchant_slist_append_unique_string (module_dirs, g_strdup (ENCHANT_GLOBAL_MODULE_DIR));
 #endif
 
 	return module_dirs;
@@ -258,19 +264,19 @@ enchant_get_conf_dirs (void)
 
 	for (iter = user_conf_dirs; iter != NULL; iter = iter->next)
 		{
-			conf_dirs = enchant_slist_append_unique_string (conf_dirs, iter->data);
+			conf_dirs = enchant_slist_append_unique_path (conf_dirs, iter->data);
 		}
 
 	g_slist_free (user_conf_dirs);
 
 #ifdef XP_TARGET_COCOA
-	conf_dirs = enchant_slist_append_unique_string (conf_dirs, g_strdup ([[EnchantResourceProvider instance] configFolder]));
+	conf_dirs = enchant_slist_append_unique_path (conf_dirs, g_strdup ([[EnchantResourceProvider instance] configFolder]));
 #endif
 
 	/* Look for explicitly set registry values */
 	ordering_dir = enchant_get_registry_value_ex (0, "Config", "Data_Dir");
 	if (ordering_dir)
-		conf_dirs = enchant_slist_append_unique_string (conf_dirs, ordering_dir);
+		conf_dirs = enchant_slist_append_unique_path (conf_dirs, ordering_dir);
 
 	/* Dynamically locate library and search for files relative to it. */
 	prefix = enchant_get_prefix_dir();
@@ -278,11 +284,11 @@ enchant_get_conf_dirs (void)
 		{
 			ordering_dir = g_build_filename(prefix,"share","enchant",NULL);
 			g_free(prefix);
-			conf_dirs = enchant_slist_append_unique_string (conf_dirs, ordering_dir);
+			conf_dirs = enchant_slist_append_unique_path (conf_dirs, ordering_dir);
 		}
 
 #if defined(ENCHANT_GLOBAL_ORDERING)
-	conf_dirs = enchant_slist_append_unique_string (conf_dirs, g_strdup (ENCHANT_GLOBAL_ORDERING));
+	conf_dirs = enchant_slist_append_unique_path (conf_dirs, g_strdup (ENCHANT_GLOBAL_ORDERING));
 #endif
 
 	return conf_dirs;
@@ -346,7 +352,7 @@ enchant_get_user_config_dirs (void)
 
 	user_config = enchant_get_registry_value_ex (1, "Config", "Data_Dir");
 	if (user_config)
-		dirs = enchant_slist_prepend_unique_string (dirs, user_config);
+		dirs = enchant_slist_prepend_unique_path (dirs, user_config);
 
 	return dirs;
 }
@@ -388,6 +394,7 @@ enchant_get_registry_value_ex (int current_user, const char * const prefix, cons
 					wszValue = g_new0(WCHAR, dwSize + 1);
 					RegQueryValueExW(hKey, uKey, NULL, &lType, (LPBYTE) wszValue, &dwSize);
 				}
+			RegCloseKey(hKey);
 		}
 
 	if(wszValue && *wszValue)
@@ -1270,7 +1277,6 @@ enchant_dict_free_string_list (EnchantDict * dict, char **string_list)
 	EnchantSession * session;
 
 	g_return_if_fail (dict);
-	g_return_if_fail (string_list);
 	session = ((EnchantDictPrivateData*)dict->enchant_private_data)->session;
 	enchant_session_clear_error (session);
 	g_strfreev(string_list);
@@ -1417,6 +1423,10 @@ enchant_load_providers_in_dir (EnchantBroker * broker, const char *dir_name)
 			if ((entry_len > g_module_suffix_len) && 
 				!strcmp(dir_entry+(entry_len-g_module_suffix_len), G_MODULE_SUFFIX))
 				{
+#ifdef _WIN32
+					/* Suppress error popups for failing to load plugins */
+					UINT old_error_mode = SetErrorMode(SEM_FAILCRITICALERRORS);
+#endif
 					filename = g_build_filename (dir_name, dir_entry, NULL);
 					
 					module = g_module_open (filename, (GModuleFlags) 0);
@@ -1451,6 +1461,10 @@ enchant_load_providers_in_dir (EnchantBroker * broker, const char *dir_name)
 						}
 					
 					g_free (filename);
+#ifdef _WIN32
+					/* Restore the original error mode */
+					SetErrorMode(old_error_mode);
+#endif
 				}
 			if (provider)
 				{
@@ -1610,11 +1624,13 @@ enchant_dict_destroyed (gpointer data)
 	EnchantDict *dict;
 	EnchantProvider *owner;
 	EnchantSession *session;
+	EnchantDictPrivateData *enchant_dict_private_data;
 	
 	g_return_if_fail (data);
 	
 	dict = (EnchantDict *) data;
-	session = ((EnchantDictPrivateData*)dict->enchant_private_data)->session;
+	enchant_dict_private_data = (EnchantDictPrivateData*)dict->enchant_private_data;
+	session = enchant_dict_private_data->session;
 	owner = session->provider;
 	
 	if (owner && owner->dispose_dict) 
@@ -1622,6 +1638,8 @@ enchant_dict_destroyed (gpointer data)
 	else if(session->is_pwl)
 		g_free (dict);
 
+	g_free(enchant_dict_private_data);
+
 	enchant_session_destroy (session);
 }
 
@@ -1719,7 +1737,7 @@ enchant_broker_free (EnchantBroker * broker)
  * @pwl: A non-null pathname in the GLib file name encoding (UTF-8 on Windows)
  *       to the personal wordlist file
  *
- * Returns: 
+ * Returns: An EnchantDict. This dictionary is reference counted.
  */
 ENCHANT_MODULE_EXPORT (EnchantDict *)
 enchant_broker_request_pwl_dict (EnchantBroker * broker, const char *const pwl)
@@ -1814,7 +1832,7 @@ _enchant_broker_request_dict (EnchantBroker * broker, const char *const tag)
  * @broker: A non-null #EnchantBroker
  * @tag: The non-null language tag you wish to request a dictionary for ("en_US", "de_DE", ...)
  *
- * Returns: An #EnchantDict, or %null if no suitable dictionary could be found.
+ * Returns: An #EnchantDict, or %null if no suitable dictionary could be found. This dictionary is reference counted.
  */
 ENCHANT_MODULE_EXPORT (EnchantDict *)
 enchant_broker_request_dict (EnchantBroker * broker, const char *const tag)
@@ -2312,3 +2330,8 @@ enchant_get_dirs_from_param (EnchantBroker * broker, const char * const param_na
 
 	return dirs;
 }
+
+ENCHANT_MODULE_EXPORT(char *)
+enchant_get_version (void) {
+	return ENCHANT_VERSION_STRING;
+}
