@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
  * gives permission to link the code of this program with
@@ -60,11 +60,14 @@
 #include <fcntl.h>
 
 #ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
 #include <io.h>
 #endif
 
 #include <glib.h>
 #include <glib/gstdio.h>
+#include "enchant-provider.h"
 
 #include "pwl.h"
 
@@ -197,7 +200,13 @@ enchant_lock_file (FILE * f)
 #elif defined(HAVE_LOCKF)
 	lockf (fileno (f), F_LOCK, 0);
 #elif defined(_WIN32)
-	_lock_file(f);
+	OVERLAPPED overlapped;
+
+	overlapped.Offset = 0;
+	overlapped.OffsetHigh = 0;
+	overlapped.hEvent = NULL;
+	if (!LockFileEx ((HANDLE) _get_osfhandle (fileno (f)), LOCKFILE_EXCLUSIVE_LOCK, 0, 0, 0x80000000, &overlapped))
+		g_warning ("Could not lock file\n");
 #else
 	/* TODO: UNIX fcntl. This race condition probably isn't too bad. */
 #endif /* HAVE_FLOCK */
@@ -211,7 +220,13 @@ enchant_unlock_file (FILE * f)
 #elif defined(HAVE_LOCKF)
 	lockf (fileno (f), F_ULOCK, 0);
 #elif defined(_WIN32)
-	_unlock_file(f);
+	OVERLAPPED overlapped;
+
+	overlapped.Offset = 0;
+	overlapped.OffsetHigh = 0;
+	overlapped.hEvent = NULL;
+	if (!UnlockFileEx ((HANDLE) _get_osfhandle (fileno (f)), 0, 0, 0x80000000, &overlapped))
+		g_warning ("Could not unlock file\n");
 #else
 	/* TODO: UNIX fcntl. This race condition probably isn't too bad. */
 #endif /* HAVE_FLOCK */
@@ -244,17 +259,17 @@ EnchantPWL* enchant_pwl_init(void)
  */ 
 EnchantPWL* enchant_pwl_init_with_file(const char * file)
 {
-	int fd;
+	FILE* fd;
 	EnchantPWL *pwl;
 
 	g_return_val_if_fail (file != NULL, NULL);
 
-	fd = g_open(file, O_CREAT | O_RDONLY, S_IREAD | S_IWRITE);
-	if(fd == -1)
+	fd = enchant_fopen(file, "ab+");
+	if(fd == NULL)
 		{
 			return NULL;
 		}
-	close(fd);
+	fclose(fd);
 	pwl = enchant_pwl_init();
 	pwl->filename = g_strdup(file);
 	pwl->file_changed = 0;
@@ -285,7 +300,7 @@ static void enchant_pwl_refresh_from_file(EnchantPWL* pwl)
 	g_hash_table_destroy (pwl->words_in_trie);
 	pwl->words_in_trie = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	f = g_fopen(pwl->filename, "r");
+	f = enchant_fopen(pwl->filename, "r");
 	if (!f) 
 		return;
 
@@ -379,7 +394,7 @@ void enchant_pwl_add(EnchantPWL *pwl,
 	{
 		FILE *f;
 		
-		f = g_fopen(pwl->filename, "a");
+		f = enchant_fopen(pwl->filename, "a");
 		if (f)
 			{
 				struct stat stats;
@@ -388,8 +403,11 @@ void enchant_pwl_add(EnchantPWL *pwl,
 				if(g_stat(pwl->filename, &stats)==0)
 					pwl->file_changed = stats.st_mtime;
 
-				fwrite (word, sizeof(char), len, f);
+                /* we write the new line first since we can't guarantee
+                   that the file was terminated by a new line before
+                   and we are just appending to the end of the file */
 				fwrite ("\n", sizeof(char), 1, f);
+				fwrite (word, sizeof(char), len, f);
 				enchant_unlock_file (f);
 				fclose (f);
 			}	
@@ -416,7 +434,7 @@ void enchant_pwl_remove(EnchantPWL *pwl,
 			if(!g_file_get_contents(pwl->filename, &contents, &length, NULL))
 				return;
 
-			f = g_fopen(pwl->filename, "wb"); /*binary because g_file_get_contents reads binary*/
+			f = enchant_fopen(pwl->filename, "wb"); /*binary because g_file_get_contents reads binary*/
 			if (f)
 				{
 					const gunichar BOM = 0xfeff;
@@ -1231,6 +1249,8 @@ static int edit_dist(const char* utf8word1, const char* utf8word2)
 	}
 
 	cost = table[len1*(len2+1) + len2];
+	g_free(word1);
+	g_free(word2);
 	g_free(table);
 	return cost;
 }
