@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Boston, MA 02110-1301, USA.
  *
  * In addition, as a special exception, Dom Lachowicz
  * gives permission to link the code of this program with
@@ -50,29 +50,75 @@ ENCHANT_PLUGIN_DECLARE("Uspell")
 static const size_t MAXALTERNATIVE = 20; // we won't return more than this number of suggestions
 static const size_t MAXCHARS = 100; // maximum number of bytes of utf8 or chars of UCS4 in a word
 
-static char *
-uspell_checker_get_prefix (void)
+static GSList *
+uspell_checker_get_dictionary_dirs (EnchantBroker * broker)
 {
-	char * data_dir = NULL;
+	GSList *dirs = NULL;
 
-	data_dir = enchant_get_registry_value ("Uspell", "Data_Dir");
-	if (data_dir)
-		return data_dir;
+	{
+		GSList *config_dirs, *iter;
+
+		config_dirs = enchant_get_user_config_dirs ();
+		
+		for (iter = config_dirs; iter; iter = iter->next)
+			{
+				dirs = g_slist_append (dirs, g_build_filename ((const gchar *)iter->data, 
+									       "uspell", NULL));
+			}
+
+		g_slist_foreach (config_dirs, (GFunc)g_free, NULL);
+		g_slist_free (config_dirs);
+	}
+
+	{
+		const gchar* const * system_data_dirs = g_get_system_data_dirs ();
+		const gchar* const * iter;
+
+		for (iter = system_data_dirs; *iter; iter++)
+			{
+				dirs = g_slist_append (dirs, g_build_filename (*iter, "uspell", "dicts", NULL));
+			}
+	}
+
+	/* until I work out how to link the modules against enchant in MacOSX - fjf
+	 */
+#ifndef XP_TARGET_COCOA
+	char * uspell_prefix = NULL;
+
+	/* Look for explicitly set registry values */
+	uspell_prefix = enchant_get_registry_value ("Uspell", "Data_Dir");
+	if (uspell_prefix)
+		dirs = g_slist_append (dirs, uspell_prefix);
 
 	/* Dynamically locate library and search for modules relative to it. */
 	char * enchant_prefix = enchant_get_prefix_dir();
 	if(enchant_prefix)
 		{
-			char * uspell_prefix = g_build_filename(enchant_prefix, "share", "enchant", "uspell", NULL);
+			uspell_prefix = g_build_filename(enchant_prefix, "share", "enchant", "uspell", NULL);
 			g_free(enchant_prefix);
-			return uspell_prefix;
+			dirs = g_slist_append (dirs, uspell_prefix);
 		}
+#endif
 
 #ifdef ENCHANT_USPELL_DICT_DIR
-	return g_strdup (ENCHANT_USPELL_DICT_DIR);
-#else
-	return NULL;
+	dirs = g_slist_append (dirs, g_strdup (ENCHANT_USPELL_DICT_DIR));
 #endif
+
+	{
+		GSList *config_dirs, *iter;
+
+		config_dirs = enchant_get_dirs_from_param (broker, "enchant.uspell.dictionary.path");
+		
+		for (iter = config_dirs; iter; iter = iter->next)
+			{
+				dirs = g_slist_append (dirs, g_strdup ((const gchar *)iter->data));
+			}
+
+		g_slist_foreach (config_dirs, (GFunc)g_free, NULL);
+		g_slist_free (config_dirs);
+	}
+
+	return dirs;
 }
 
 static int
@@ -134,7 +180,8 @@ uspell_dict_suggest (EnchantDict * me, const char *const word,
 	char **sugg_arr = NULL;
 	const utf8_t *sugg;
 	wide_t buf[MAXCHARS];
-	int length, i;
+	int length;
+  unsigned int i;
 	utf8_t **list;
 	
 	if (len >= MAXCHARS) // no suggestions; the word is outlandish
@@ -191,8 +238,8 @@ uspell_dict_add_to_session (EnchantDict * me, const char *const word,
 } // uspell_dict_add_to_session
 
 typedef struct {
-	char * language_tag;
-	char * corresponding_uspell_file_name;
+	const char * language_tag;
+	const char * corresponding_uspell_file_name;
 	int language_flags;
 } Mapping;
 
@@ -206,7 +253,7 @@ static const Mapping mapping [] = {
 static const size_t n_mappings = (sizeof(mapping)/sizeof(mapping[0]));
 
 static void
-s_buildHashNames (std::vector<std::string> & names, const char * tag)
+s_buildHashNames (std::vector<std::string> & names, EnchantBroker * broker, const char * tag)
 {
 	names.clear ();
 
@@ -219,31 +266,23 @@ s_buildHashNames (std::vector<std::string> & names, const char * tag)
 
 	if (mapIndex < n_mappings) {
 
-		char * tmp, * private_dir, * home_dir, * uspell_prefix;
-		
-		char * dict = mapping[mapIndex].language_tag;
+		GSList *dirs, *iter;
+		char * dict = g_strdup_printf ("%s.uspell.dat",
+					       mapping[mapIndex].corresponding_uspell_file_name);
 
-		home_dir = enchant_get_user_home_dir ();
-		
-		if (home_dir) {
-			private_dir = g_build_filename (home_dir, ".enchant", 
-							"uspell", NULL);
-			
-			tmp = g_build_filename (private_dir, dict, NULL);
-			names.push_back (tmp);
-			g_free (tmp);
-			
-			g_free (private_dir);
-			g_free (home_dir);
-		}
-		
-		uspell_prefix = uspell_checker_get_prefix ();
-		if (uspell_prefix) {
-			tmp = g_build_filename (uspell_prefix, dict, NULL);
-			names.push_back (tmp);
-			g_free (tmp);
-			g_free (uspell_prefix);
-		}
+		dirs = uspell_checker_get_dictionary_dirs (broker);
+
+		for (iter = dirs; iter; iter = iter->next)
+			{
+				char *tmp;
+
+				tmp = g_build_filename ((const char *)iter->data, dict, NULL);
+				names.push_back (tmp);
+				g_free (tmp);
+			}
+
+		g_slist_foreach (dirs, (GFunc)g_free, NULL);
+		g_slist_free (dirs);
 	}
 }
 
@@ -278,89 +317,63 @@ uspell_request_dict (const char * base, const char * mapping, const int flags)
 }
 
 static uSpell *
-uspell_request_manager (const char * private_dir, size_t mapIndex)
+uspell_request_manager (const char * dir, size_t mapIndex)
 {
-	char * uspell_prefix, * home_dir;
-
 	uSpell * manager = NULL;
 
-	manager = uspell_request_dict (private_dir,
+	manager = uspell_request_dict (dir,
 				       mapping[mapIndex].corresponding_uspell_file_name,
 				       mapping[mapIndex].language_flags);
 
-	if (!manager) {
-		uspell_prefix = uspell_checker_get_prefix ();
-
-		if (uspell_prefix) {
-			manager = uspell_request_dict (uspell_prefix,
-						       mapping[mapIndex].corresponding_uspell_file_name,
-						       mapping[mapIndex].language_flags);
-			g_free (uspell_prefix);
-		}
-	}
-
 	if (!manager) return NULL;
+
 	// look for a supplementary private dictionary
-	home_dir = enchant_get_user_home_dir ();
-	if (home_dir) {
+	const char *config_dir = g_get_user_config_dir();
+	if (config_dir) {
 		gchar * auxFileName, * transPart;
 		transPart = g_strconcat (mapping[mapIndex].language_tag, ".dic", NULL);
-		auxFileName = g_build_filename (home_dir, ".enchant", transPart, NULL);
+		auxFileName = g_build_filename (config_dir, transPart, NULL);
 		g_free (transPart);
-		g_free (home_dir);
 
 		(void) manager->assimilateFile (auxFileName);
 		g_free (auxFileName);
 	}
+
 	return manager;
 }
 
+extern "C" {
+
+ENCHANT_MODULE_EXPORT (EnchantProvider *) 
+	     init_enchant_provider (void);
+
 static EnchantDict *
 uspell_provider_request_dict (EnchantProvider * me, const char *const tag)
 {
 	EnchantDict *dict = NULL;
 	uSpell *manager = NULL;
-	int mapIndex;
+	size_t mapIndex;
+	bool found = false;
 
-	char * private_dir = NULL, * home_dir;
+	GSList *dirs, *iter;
 
-	home_dir = enchant_get_user_home_dir ();
-
-	if (home_dir) {
-		private_dir = g_build_filename (home_dir, ".enchant",
-						"uspell", NULL);
-		g_free (home_dir);
-	}
-
-	for (mapIndex = 0; mapIndex < n_mappings; mapIndex++) {
+	for (mapIndex = 0; mapIndex < n_mappings && !found; mapIndex++) {
 		if (!strcmp(tag, mapping[mapIndex].language_tag)) 
 			break;
 	}
 
-	if (mapIndex < n_mappings) {
-		manager = uspell_request_manager (private_dir, mapIndex);
-	}
+	if (!found)
+		return NULL;
 
-	if (!manager) {
-		// try shortened form: he_IL => he
-		std::string shortened_dict (tag);
-		size_t uscore_pos;
-		
-		if ((uscore_pos = shortened_dict.rfind ('_')) != ((size_t)-1)) {
-			shortened_dict = shortened_dict.substr(0, uscore_pos);
+	dirs = uspell_checker_get_dictionary_dirs (me->owner);
 
-			for (mapIndex = 0; mapIndex < n_mappings; mapIndex++) {
-				if (!strcmp(shortened_dict.c_str(), mapping[mapIndex].language_tag)) 
-					break;
-			}
-
-			if (mapIndex < n_mappings) {
-				manager = uspell_request_manager (private_dir, mapIndex);
-			}			
+	for (iter = dirs; iter && !manager; iter = iter->next)
+		{
+			manager = uspell_request_manager ((const char *)iter->data, mapIndex);
 		}
-	}
-
-	g_free (private_dir);
+	
+	g_slist_foreach (dirs, (GFunc)g_free, NULL);
+	g_slist_free (dirs);
 
 	if (!manager) 
 		return NULL;
@@ -381,25 +394,12 @@ uspell_provider_dictionary_exists (struct str_enchant_provider * me,
 {
 	std::vector <std::string> names;
 
-	s_buildHashNames (names, tag);
+	s_buildHashNames (names, me->owner, tag);
 	for (size_t i = 0; i < names.size(); i++) {
 		if (g_file_test (names[i].c_str(), G_FILE_TEST_EXISTS))
 			return 1;
 	}
 
-	std::string shortened_dict (tag);
-	size_t uscore_pos;
-	
-	if ((uscore_pos = shortened_dict.rfind ('_')) != ((size_t)-1)) {
-		shortened_dict = shortened_dict.substr(0, uscore_pos);
-
-		s_buildHashNames (names, shortened_dict.c_str());
-		for (size_t i = 0; i < names.size(); i++) {
-			if (g_file_test (names[i].c_str(), G_FILE_TEST_EXISTS))
-				return 1;
-		}
-	}
-
 	return 0;
 }
 
@@ -447,21 +447,19 @@ uspell_provider_dispose (EnchantProvider * me)
 	g_free (me);
 }
 
-static char *
+static const char *
 uspell_provider_identify (EnchantProvider * me)
 {
 	return "uspell";
 }
 
-static char *
+static const char *
 uspell_provider_describe (EnchantProvider * me)
 {
 	return "Uspell Provider";
 }
 
-extern "C" {
-
-ENCHANT_MODULE_EXPORT (EnchantProvider *) 
+EnchantProvider *
 init_enchant_provider (void)
 {
 	EnchantProvider *provider;
